package main

import (
	"bytes"
	"fmt"
	"git.swzry.com/zry/mz700-tape-wav-gen"
	"github.com/urfave/cli/v2"
	"io/ioutil"
)

func ConvertBetweenASCIIandMZ(context *cli.Context, asc2mz bool, rf bool) error {
	fnin := context.String("in")
	fnout := context.String("out")
	ign := context.Bool("ignore-non-disp-char")
	din, err := ioutil.ReadFile(fnin)
	if err != nil {
		return fmt.Errorf("failed read input file: %s", err.Error())
	}
	var dout []byte
	if asc2mz {
		dout = _ASCII2MZ700(din, ign, rf)
	} else {
		dout = _MZ7002ASCII(din, ign, rf)
	}
	err = ioutil.WriteFile(fnout, dout, 0666)
	if err != nil {
		return fmt.Errorf("failed write to output file: %s", err.Error())
	}
	return nil
}

func _ASCII2MZ700(in []byte, ign bool, ir bool) []byte {
	db := make([]byte, 0, len(in))
	bb := bytes.NewBuffer(db)
	for _, b := range in {
		if b == 0x0A {
			bb.WriteByte(0x0D)
			continue
		}
		if b == 0x0D && ir {
			continue
		}
		if !ign || b >= 0x20 && b <= 0x7e {
			cvb := mz700_tape_wav_gen.ASCII_TO_MZ700_TABLE[b]
			if cvb == 0 {
				bb.WriteByte(b)
			} else {
				bb.WriteByte(cvb)
			}
		}
	}
	return bb.Bytes()
}

func _MZ7002ASCII(in []byte, ign bool, ir bool) []byte {
	db := make([]byte, 0, len(in))
	bb := bytes.NewBuffer(db)
	for _, b := range in {
		if b == 0x0D {
			if ir {
				bb.WriteByte(0x0D)
			}
			bb.WriteByte(0x0A)
			continue
		}
		if b >= 0x20 && b <= 0x5D {
			bb.WriteByte(b)
			continue
		}
		cvb := mz700_tape_wav_gen.ASCII_TO_MZ700_TABLE[b]
		if !ign || (cvb >= 0x20 && cvb <= 0x7E) {
			bb.WriteByte(cvb)
		}
	}
	return bb.Bytes()
}
