package main

import (
	"fmt"
	"github.com/urfave/cli/v2"
	"gopkg.in/yaml.v2"
	"io/ioutil"
)

func CmdSimple(context *cli.Context) error {
	ydat, err := SimpleArgsToYAMLData(context)
	if err != nil {
		return fmt.Errorf("failed parse args: %s", err.Error())
	}
	verbose := context.Bool("verbose")
	if verbose {
		yb, err := yaml.Marshal(ydat)
		if err != nil {
			return fmt.Errorf("failed marshal yaml: %s", err.Error())
		}
		ys := string(yb)
		fmt.Println("==== Processing Config ====")
		fmt.Println(ys)
	}
	fmt.Println("==== Starting Converting ====")
	err = ProcessingConvert(ydat, verbose)
	if err != nil {
		return fmt.Errorf("failed do converting: %s", err.Error())
	}
	return nil
}

func CmdYamlConfig(context *cli.Context) error {
	ifn := context.String("in")
	if ifn == "" {
		return fmt.Errorf("no input file specified")
	}
	fdata, err := ioutil.ReadFile(ifn)
	if err != nil {
		return fmt.Errorf("failed read yaml file: %s", err.Error())
	}
	var ydat ConfigDef
	err = yaml.Unmarshal(fdata, &ydat)
	if err != nil {
		return fmt.Errorf("failed parse yaml: %s", err.Error())
	}
	verbose := context.Bool("verbose")
	fmt.Println("==== Starting Converting ====")
	err = ProcessingConvert(&ydat, verbose)
	if err != nil {
		return fmt.Errorf("failed do converting: %s", err.Error())
	}
	return nil
}

func CmdExampleYaml(context *cli.Context) error {
	ydat, err := SimpleArgsToYAMLData(context)
	youtf := context.String("yaml")
	if err != nil {
		return fmt.Errorf("failed parse args: %s", err.Error())
	}
	yb, err := yaml.Marshal(ydat)
	if err != nil {
		return fmt.Errorf("failed marshal yaml: %s", err.Error())
	}
	verbose := context.Bool("verbose")
	if verbose {
		ys := string(yb)
		fmt.Println("==== YAML Content ====")
		fmt.Println(ys)
		fmt.Println("========")
	}
	err = ioutil.WriteFile(youtf, yb, 0666)
	if err != nil {
		return fmt.Errorf("failed write out yaml file: %s", err.Error())
	}
	fmt.Println("Output: ", youtf)
	return nil
}

func CmdConvertASCII2MZ(context *cli.Context) error {
	rf := context.Bool("ignore-return")
	return ConvertBetweenASCIIandMZ(context, true, rf)
}

func CmdConvertMZ2ASCII(context *cli.Context) error {
	rf := context.Bool("use-cr-lf")
	return ConvertBetweenASCIIandMZ(context, false, rf)
}
