package main

import (
	"fmt"
	"github.com/urfave/cli/v2"
)

func SimpleArgsToYAMLData(context *cli.Context) (*ConfigDef, error) {
	sla := context.String("load-addr")
	u64la, err := ParseUInt16WithMultiBase(sla)
	if err != nil {
		return nil, fmt.Errorf("failed parse arg 'load-addr': %s", err.Error())
	}
	var exec_after_load bool
	var exec_addr uint16
	sea := context.String("exec-addr")
	if sea == "" {
		exec_after_load = false
	} else {
		exec_after_load = true
		u64ea, err := ParseUInt16WithMultiBase(sea)
		if err != nil {
			return nil, fmt.Errorf("failed parse arg 'exec-addr': %s", err.Error())
		}
		exec_addr = u64ea
	}
	binf := context.String("in")
	ydata := &ConfigDef{
		OutputWav:      context.String("out"),
		MachineType:    context.String("machine"),
		InvertPolarity: context.Bool("invert-polarity"),
		TapeFiles: []ConfigItemTapeFile{
			{
				FileAttribute:           context.String("file-attrib"),
				FileName:                context.String("file-name"),
				LoadAddress:             u64la,
				ExecuteAfterLoad:        exec_after_load,
				ExecuteAddress:          exec_addr,
				UseRawCommentFromFile:   false,
				RawCommentFileName:      "",
				StringComment:           context.String("comment"),
				DateSource:              "bin-file",
				FileDataSource:          binf,
				ASCIITextFileDataSource: "",
				IgnoreNonDisplayChar:    false,
				RawData:                 nil,
			},
		},
	}
	return ydata, nil
}
