package main

import (
	"bytes"
	"fmt"
	"strconv"
	"strings"
)

func ParseUInt16WithMultiBase(str string) (uint16, error) {
	nstr := str
	base := 10
	fld := false
	if strings.HasPrefix(nstr, "0x") {
		fld = true
		nstr = strings.TrimPrefix(nstr, "0x")
		base = 16
	}
	if strings.HasSuffix(nstr, "h") {
		if fld {
			return 0, fmt.Errorf("do not use multiple base indicator")
		}
		fld = true
		nstr = strings.TrimSuffix(nstr, "h")
		base = 16
	}
	if strings.HasSuffix(nstr, "H") {
		if fld {
			return 0, fmt.Errorf("do not use multiple base indicator")
		}
		fld = true
		nstr = strings.TrimSuffix(nstr, "H")
		base = 16
	}
	if strings.HasSuffix(nstr, "o") {
		if fld {
			return 0, fmt.Errorf("do not use multiple base indicator")
		}
		fld = true
		nstr = strings.TrimSuffix(nstr, "o")
		base = 8
	}
	if strings.HasSuffix(nstr, "O") {
		if fld {
			return 0, fmt.Errorf("do not use multiple base indicator")
		}
		fld = true
		nstr = strings.TrimSuffix(nstr, "O")
		base = 8
	}
	if strings.HasSuffix(nstr, "b") {
		if fld {
			return 0, fmt.Errorf("do not use multiple base indicator")
		}
		fld = true
		nstr = strings.TrimSuffix(nstr, "b")
		base = 2
	}
	if strings.HasSuffix(nstr, "B") {
		if fld {
			return 0, fmt.Errorf("do not use multiple base indicator")
		}
		fld = true
		nstr = strings.TrimSuffix(nstr, "B")
		base = 2
	}
	u64la, err := strconv.ParseUint(nstr, base, 16)
	if err != nil {
		return 0, err
	}
	return uint16(u64la), nil
}

func ParseFileAttrib(str string) (byte, string, error) {
	b, err := strconv.ParseUint(str, 16, 8)
	if err != nil {
		return 0, "", fmt.Errorf("failed parse file attribute: %s", err.Error())
	}
	s := "UNKNOWN"
	switch b {
	case 0x01:
		s = "MACHINE CODE PROGRAM FILE"
		break
	case 0x02:
		s = "MZ-80 BASIC PROGRAM FILE"
		break
	case 0x03:
		s = "MZ-80 DATA FILE"
		break
	case 0x04:
		s = "MZ-700 DATA FILE"
		break
	case 0x05:
		s = "MZ-700 BASIC PROGRAM FILE"
		break
	default:
		s = "UNKNOWN"
		break
	}
	return byte(b), s, nil
}

func AddTabBeforeEachLine(in string) string {
	lines := strings.Split(in, "\n")
	strwr := bytes.NewBufferString("")
	for _, v := range lines {
		strwr.WriteRune('\t')
		strwr.WriteString(v)
		strwr.WriteRune('\n')
	}
	return strwr.String()
}
